==> swim-http-4.0.1-sources.jar: swim/http/ParamListWriter.java <==
// Copyright 2015-2021 Swim Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package swim.http;

import java.util.Iterator;
import swim.codec.Output;
import swim.codec.Writer;
import swim.codec.WriterException;

final class ParamListWriter extends Writer<Object, Object> {

  final HttpWriter http;
  final Iterator<? extends HttpPart> params;
  final Writer<?, ?> paramWriter;
  final int step;

  ParamListWriter(HttpWriter http, Iterator<? extends HttpPart> params,
                  Writer<?, ?> paramWriter, int step) {
    this.http = http;
    this.params = params;
    this.paramWriter = paramWriter;
    this.step = step;
  }

  ParamListWriter(HttpWriter http, Iterator<? extends HttpPart> params) {
    this(http, params, null, 1);
  }

  @Override
  public Writer<Object, Object> pull(Output<?> output) {
    return ParamListWriter.write(output, this.http, this.params,
                                 this.paramWriter, this.step);
  }

  static Writer<Object, Object> write(Output<?> output, HttpWriter http,
                                      Iterator<? extends HttpPart> params,
                                      Writer<?, ?> paramWriter, int step) {
    do {
      if (step == 1) {
        if (paramWriter == null) {
          if (!params.hasNext()) {
            return Writer.done();
          } else {
            paramWriter = params.next().writeHttp(output, http);
          }
        } else {
          paramWriter = paramWriter.pull(output);
        }
        if (paramWriter.isDone()) {
          paramWriter = null;
          if (!params.hasNext()) {
            return Writer.done();
          } else {
            step = 2;
          }
        } else if (paramWriter.isError()) {
          return paramWriter.asError();
        }
      }
      if (step == 2 && output.isCont()) {
        output = output.write(',');
        step = 3;
      }
      if (step == 3 && output.isCont()) {
        output = output.write(' ');
        step = 1;
        continue;
      }
      break;
    } while (true);
    if (output.isDone()) {
      return Writer.error(new WriterException("truncated"));
    } else if (output.isError()) {
      return Writer.error(output.trap());
    }
    return new ParamListWriter(http, params, paramWriter, step);
  }

  static Writer<Object, Object> write(Output<?> output, HttpWriter http,
                                      Iterator<? extends HttpPart> params) {
    return ParamListWriter.write(output, http, params, null, 1);
  }

}

==> swim-http-4.2.14-sources.jar: swim/http/ParamListWriter.java <==
// Copyright 2015-2023 Nstream, inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package swim.http;

import java.util.Iterator;
import swim.codec.Output;
import swim.codec.Writer;
import swim.codec.WriterException;

final class ParamListWriter extends Writer<Object, Object> {

  final HttpWriter http;
  final Iterator<? extends HttpPart> params;
  final Writer<?, ?> paramWriter;
  final int step;
  final char separator;

  ParamListWriter(HttpWriter http, Iterator<? extends HttpPart> params, char separator,
                  Writer<?, ?> paramWriter, int step) {
    this.http = http;
    this.params = params;
    this.paramWriter = paramWriter;
    this.step = step;
    this.separator = separator;
  }

  ParamListWriter(HttpWriter http, Iterator<? extends HttpPart> params, char separator) {
    this(http, params, separator, null, 1);
  }

  @Override
  public Writer<Object, Object> pull(Output<?> output) {
    return ParamListWriter.write(output, this.http, this.params, this.separator,
         this.paramWriter, this.step);
  }

  static Writer<Object, Object> write(Output<?> output, HttpWriter http,
                                      Iterator<? extends HttpPart> params, char separator,
                                      Writer<?, ?> paramWriter, int step) {
    do {
      if (step == 1) {
        if (paramWriter == null) {
          if (!params.hasNext()) {
            return Writer.done();
          } else {
            paramWriter = params.next().writeHttp(output, http);
          }
        } else {
          paramWriter = paramWriter.pull(output);
        }
        if (paramWriter.isDone()) {
          paramWriter = null;
          if (!params.hasNext()) {
            return Writer.done();
          } else {
            step = 2;
          }
        } else if (paramWriter.isError()) {
          return paramWriter.asError();
        }
      }
      if (step == 2 && output.isCont()) {
        output = output.write(separator);
        step = 3;
      }
      if (step == 3 && output.isCont()) {
        output = output.write(' ');
        step = 1;
        continue;
      }
      break;
    } while (true);
    if (output.isDone()) {
      return Writer.error(new WriterException("truncated"));
    } else if (output.isError()) {
      return Writer.error(output.trap());
    }
    return new ParamListWriter(http, params, separator, paramWriter, step);
  }

  static Writer<Object, Object> write(Output<?> output, HttpWriter http,
                                      Iterator<? extends HttpPart> params, char separator) {
    return ParamListWriter.write(output, http, params, separator, null, 1);
  }

}
